require "memoizable"
require "time"

module Greentext
  # Internal: The base class for the models.
  class Base
    include Memoizable
    attr_reader :attrs
    alias_method :to_h, :attrs

    class << self
      # Internal: Dynamically define an attribute method for a given attribute
      #
      # key   - A Symbol of the method name.
      # value - An optional Symbol of the attribute key.
      #
      # Yields an optional attribute key.
      #
      # Returns nothing.
      def attr_reader(key, value=key)
        define_method(key) { @attrs[value.to_s] }
        memoize(key)
      end

      # Internal: Dynamically define a predicate method for a given attribute.
      #
      # key   - A Symbol of the method name.
      # value - An optional Symbol of the attribute key.
      #
      # Returns nothing.
      def attr_predicate(key, value=key)
        define_method(:"#{key}?") { @attrs[value.to_s] == 1 }
        memoize(:"#{key}?")
      end

      # Internal: Dynamically define an date method for a given attribute.
      #
      # key   - A Symbol of the method name.
      # value - An optional Symbol of the attribute key.
      #
      # Returns nothing.
      def attr_epoch(key, value=key)
        define_method(key) { DateTime.strptime(@attrs[value.to_s].to_s, "%s") }
        memoize(key)
      end
    end

    # Internal: Initialize a new Object with given attributes.
    #
    # attrs - An optional Hash of attributes.
    #
    # Returns an instance of the Object
    def initialize(attrs)
      @attrs = attrs
    end
  end
end
